.sidenav {                                                                      
    height: 100%;                                                               
    width: 250px;                                                               
    position: absolute;                                                         
    z-index: 1;                                                                 
    background-color: #FFFFFF;                                                  
    overflow-x: hidden;                                                         
    padding-top: 30px;                                                          
    box-shadow: 0 0 5px grey;                                                                                                   
    text-decoration: none;                                                      
    display: block;   
    padding-left: 20px;                                                          
}                                                                               
                                                                                
.close {                                                                        
  cursor: pointer;                                                              
  position: absolute;                                                           
  top: 2%;                                                                      
  right: 0%;                                                                    
  padding: 12px 16px;                                                           
  transform: translate(0%, -50%);                                               
}                                                                               
.close:hover {background: #bbb;}
